@@ -43,7 +43,7 @@ class Timer:
         """
         if self._paused is None:
             raise ValueError("Trying to resume a Timer that is not paused!")
-        self._total_paused += perf_counter() - self._paused  # pyre-ignore
+        self._total_paused += perf_counter() - self._paused
         self._paused = None
         self._count_start += 1
 
