@@ -5,16 +5,22 @@ import logging
 import typing
 import warnings
 from collections import Counter
+from copy import copy
+from dataclasses import dataclass
+from numbers import Number
 from typing import Any, Dict, List, Optional, Set, Tuple, Union
 
 import torch
 import torch.nn as nn
 from fvcore.common.checkpoint import _named_modules_with_dup
+from torch import Tensor
 from torch.jit import TracerWarning, _get_trace_graph
 
 from .jit_handles import Handle
 
 
+# Only ignore ops that are technically truly 0 flops:
+# shape-manipulation ops, integer ops, memory copy ops
 _IGNORED_OPS: Set[str] = {
     "aten::Int",
     "aten::ScalarImplicit",
@@ -53,28 +59,37 @@ _IGNORED_OPS: Set[str] = {
     "aten::split",
     "aten::split_with_sizes",
     "aten::squeeze",
+    "aten::narrow",
+    "aten::unbind",
+    "aten::full_like",
     "aten::stack",
     "aten::t",
     "aten::to",
+    "aten::type_as",
     "aten::transpose",
     "aten::unsqueeze",
     "aten::unsqueeze_",
     "aten::view",
     "aten::zeros",
     "aten::zeros_like",
-    "prim::Constant",
-    "prim::ImplicitTensorToNum",
-    "prim::Int",
-    "prim::ListConstruct",
-    "prim::ListUnpack",
-    "prim::NumToTensor",
-    "prim::TupleConstruct",
 }
 
 
+@dataclass
+class Statistics:
+    """
+    For keeping track of the various model statistics recorded during
+    analysis.
+    """
+
+    counts: "Dict[str, Counter[str]]"
+    unsupported_ops: "Dict[str, Counter[str]]"
+    uncalled_mods: "Set[str]"
+
+
 def _get_scoped_trace_graph(
     module: nn.Module,
-    inputs: Tuple[object, ...],
+    inputs: Union[Tensor, Tuple[Tensor, ...]],
     aliases: Dict[Union[str, nn.Module], str],
 ) -> torch._C.Graph:  # pyre-ignore[11]
     """
@@ -95,33 +110,25 @@ def _get_scoped_trace_graph(
         graph (torch._C.Graph) : The pytorch JIT trace of the model
     """
 
-    class ScopePushHook(object):
+    class ScopePushHook:
         def __init__(self, name: str) -> None:
             self.name = name
 
-        def __call__(
-            self, module: nn.Module, inputs: Tuple[object, ...]
-        ) -> Tuple[object, ...]:
+        def __call__(self, module: nn.Module, inputs: Any) -> Any:
             tracing_state = torch._C._get_tracing_state()
             if tracing_state:
-                name = self.name
-                tracing_state.push_scope(name)
+                tracing_state.push_scope(self.name)
             return inputs
 
-    class ScopePopHook(object):
-        def __call__(
-            self,
-            module: nn.Module,
-            inputs: Tuple[object, ...],
-            outputs: Tuple[object, ...],
-        ) -> Tuple[object, ...]:
+    class ScopePopHook:
+        def __call__(self, module: nn.Module, inputs: Any, outputs: Any) -> Any:
             tracing_state = torch._C._get_tracing_state()
             if tracing_state:
                 tracing_state.pop_scope()
             return outputs
 
     seen = set()
-    hook_handles = []  # type: List[Any]
+    hook_handles: List[Any] = []
 
     def register_hooks(mod: nn.Module, name: str) -> None:
         prehook = mod.register_forward_pre_hook(ScopePushHook(name))  # pyre-ignore[16]
@@ -161,7 +168,7 @@ def _get_scoped_trace_graph(
     return graph
 
 
-class JitModelAnalysis(object):
+class JitModelAnalysis:
     """
     Provides access to per-submodule model statistics obtained by
     tracing a model with pytorch's jit tracing functionality. Calculates
@@ -183,36 +190,28 @@ class JitModelAnalysis(object):
     contributing statistics to that module.
     """
 
-    ignored_ops = _IGNORED_OPS  # type: Set[str]
-
     def __init__(
         self,
         model: nn.Module,
-        inputs: Tuple[object, ...],
-        op_handles: Optional[Dict[str, Handle]] = None,
+        inputs: Union[Tensor, Tuple[Tensor, ...]],
     ) -> None:
         """
         Args:
-            model (nn.Module) : The model to analyze
-            inputs (tuple) : The inputs to the model for analysis.
-            op_handles (dict(str,Callable) : Map an operator name in the
-                trace graph to a function used to calculate the desire
-                statistic. The function must take the inputs and outputs
-                of the op, each as a list(torch._C.Value), and returns
-                a counter of the form {op_name : number}.
+            model: The model to analyze
+            inputs: The inputs to the model for analysis.
         """
         self._model = model
         self._inputs = inputs
-        self._op_handles = (
-            op_handles if op_handles is not None else {}
-        )  # type: Dict[str, Handle]
-        self._aliases = self._get_aliases(
-            model
-        )  # type: Dict[Union[nn.Module, str], str]
-        self._counts = None  # type: Optional[Dict[str, typing.Counter[str]]]
-        self._skipped_ops = None  # type: Optional[Dict[str, typing.Counter[str]]]
-        self._warn_skipped = True
+        self._op_handles: Dict[str, Handle] = {}
+        # Mapping from names to submodules
+        self._named_modules: Dict[str, nn.Module] = dict(_named_modules_with_dup(model))
+        # Mapping from submodules and their aliases to the canonical name of each submodule
+        self._aliases: Dict[Union[nn.Module, str], str] = self._get_aliases(model)
+        self._stats: Optional[Statistics] = None
+        self._enable_warn_unsupported_ops = True
+        self._enable_warn_uncalled_mods = True
         self._warn_trace = "no_tracer_warning"
+        self._ignored_ops: Set[str] = copy(_IGNORED_OPS)
 
     def total(self, module_name: str = "") -> int:
         """
@@ -225,16 +224,16 @@ class JitModelAnalysis(object):
         Returns:
             int : The aggregated statistic.
         """
-        counts, _ = self._analyze()
+        stats = self._analyze()
         module_name = self.canonical_module_name(module_name)
-        total_count = sum(counts[module_name].values())
+        total_count = sum(stats.counts[module_name].values())
         return total_count
 
     def by_operator(self, module_name: str = "") -> typing.Counter[str]:
         """
-        Returns the statistics for a requested module, separated out by
-        operator type. The operator handle determines the name associated
-        with each operator type.
+        Returns the statistics for a requested module, grouped by operator
+        type. The operator handle determines the name associated with each
+        operator type.
 
         Args:
             module_name (str) : The submodule to get data for. Defaults
@@ -242,9 +241,9 @@ class JitModelAnalysis(object):
         Returns:
             Counter(str) : The statistics for each operator.
         """
-        counts, _ = self._analyze()
+        stats = self._analyze()
         module_name = self.canonical_module_name(module_name)
-        return counts[module_name]
+        return stats.counts[module_name]
 
     def by_module_and_operator(self) -> Dict[str, typing.Counter[str]]:
         """
@@ -253,75 +252,113 @@ class JitModelAnalysis(object):
         the name associated with each operator type.
 
         Returns:
-            dict(str, Counter(str)) : The statistics for each submodule
-                and each operator. Organized per-module, labeled
-                by the submodule's name, then by operator name.
+            dict(str, Counter(str)):
+                The statistics for each submodule and each operator.
+                Grouped by submodule names, then by operator name.
         """
-        counts, _ = self._analyze()
-        return counts
+        stats = self._analyze()
+        return stats.counts
 
     def by_module(self) -> typing.Counter[str]:
         """
         Returns the statistics for all submodules, aggregated over
-            all operators.
+        all operators.
 
         Returns:
-            Counter(str) : The statistics for each submodule
-                and each operator. Organized per-module, labeled
-                by the submodule's name.
+            Counter(str): statistics counter grouped by submodule names
         """
-        counts, _ = self._analyze()
+        stats = self._analyze()
         summed_counts = Counter()
-        for mod, results in counts.items():
+        for mod, results in stats.counts.items():
             summed_counts[mod] = sum(results.values())
         return summed_counts
 
-    def skipped_ops(self, module_name: str = "") -> typing.Counter[str]:
+    def unsupported_ops(self, module_name: str = "") -> typing.Counter[str]:
         """
-        Lists the number of operators that were skipped because no
-        operator handle existed for them. Does not include operators
-        listed in _IGNORED_OPS.
+        Lists the number of operators that were encountered but unsupported
+        because no operator handle is available for them. Does not include
+        operators that are explicitly ignored.
 
         Args:
-            module_name (str) : The submodule to skipped ops for. Defaults to
-                the entire model.
+            module_name (str) : The submodule to list unsupported ops.
+                Defaults to the entire model.
+
         Returns:
-            Counter(str) : The number of each type of operator skipped.
+            Counter(str) : The number of occurences each unsupported operator.
         """
-        _, skipped_ops = self._analyze()
+        if self._stats is None:
+            raise RuntimeError(
+                "Analysis results should be computed "
+                "before calling unsupported_ops()"
+            )
         module_name = self.canonical_module_name(module_name)
-        return skipped_ops[module_name]
+        return self._stats.unsupported_ops[module_name]  # pyre-fixme
 
-    def set_op_handle(self, name: str, func: Handle) -> None:
+    def uncalled_modules(self) -> Set[str]:
         """
-        Sets an additional operator handle, or replacing an existing one.
+        Returns a set of submodules that were never called during the
+        trace of the graph. This may be because they were unused, or
+        because they were accessed via direct calls .forward() or with
+        other python methods. In the latter case, statistics will not be
+        attributed to the submodule, though the statistics will be included
+        in the parent module.
+
+        Returns:
+            set(str) : The set of submodule names that were never called
+                during the trace of the model.
+        """
+        stats = self._analyze()
+        return stats.uncalled_mods
+
+    def set_op_handle(self, *args, **kwargs: Optional[Handle]) -> "JitModelAnalysis":
+        """
+        Sets additional operator handles, or replaces existing ones.
 
         Args:
-            name (str) : The operator's name
-            func (Callable) : Function that calculates the desirable
-                statistic from an operator. Must take two arguments,
-                which are the inputs and outputs of the operator, in the
-                form of list(torch._C.Value).
-        """
-        self._op_handles[name] = func
-        self._counts = None
-        self._skipped_ops = None
+            args: (str, Handle) pairs of operator names and handles.
+            kwargs: mapping from operator names to handles.
 
-    def clear_op_handles(self) -> None:
+        If a handle is ``None``, the op will be explicitly ignored. Otherwise,
+        handle should be a function that calculates the desirable statistic
+        from an operator. The function must take two arguments, which are the
+        inputs and outputs of the operator, in the form of ``list(torch._C.Value)``.
+        The function should return a counter object with per-operator statistics.
+
+        Examples
+        ::
+            handlers = {"aten::linear": my_handler}
+            counter.set_op_handle("aten::matmul", None, "aten::bmm", my_handler2)
+                   .set_op_handle(**handlers)
         """
-        Clears all set operator handles.
+        self._stats = None
+        if len(args) % 2 != 0:
+            raise TypeError(
+                "set_op_handle should be called with pairs of names and handles!"
+            )
+        for name, handle in zip(args[::2], args[1::2]):
+            kwargs[name] = handle
+        for name, handle in kwargs.items():
+            if handle is None:
+                self._ignored_ops.add(name)
+            else:
+                self._op_handles[name] = handle
+        return self
+
+    def clear_op_handles(self) -> "JitModelAnalysis":
+        """
+        Clears all operator handles currently set.
         """
         self._op_handles = {}
-        self._counts = None
-        self._skipped_ops = None
+        self._ignored_ops = copy(_IGNORED_OPS)
+        self._stats = None
+        return self
 
     def canonical_module_name(self, name: str) -> str:
         """
-        Returns the canonical module name of the module or module name.
+        Returns the canonical module name of the given ``name``, which might be
+        different from the given ``name`` if the module is shared.
         This is the name that will be used as a key when statistics are
-        output using .by_module() and .by_module_and_operator(). It is
-        the first name encountered for a module when walking the descendants
-        of the model.
+        output using .by_module() and .by_module_and_operator().
 
         Args:
             name (str) : The name of the module to find the canonical name for.
@@ -341,11 +378,11 @@ class JitModelAnalysis(object):
     def copy(
         self,
         new_model: Optional[nn.Module] = None,
-        new_inputs: Optional[Tuple[object, ...]] = None,
+        new_inputs: Union[None, Tensor, Tuple[Tensor, ...]] = None,
     ) -> "JitModelAnalysis":
         """
-        Returns a copy of the JitModelAnalysis object, keeping all
-        settings, but potentially on a new model or new inputs.
+        Returns a copy of the :class:`JitModelAnalysis` object, keeping all
+        settings, but on a new model or new inputs.
 
         Args:
             new_model (nn.Module or None) : a new model for the new
@@ -358,26 +395,26 @@ class JitModelAnalysis(object):
         """
         model = self._model if new_model is None else new_model
         inputs = self._inputs if new_inputs is None else new_inputs
-        analyzer = JitModelAnalysis(
-            model=model, inputs=inputs, op_handles=self._op_handles
+        return (
+            JitModelAnalysis(model=model, inputs=inputs)
+            .set_op_handle(**self._op_handles)
+            .unsupported_ops_warnings(self._enable_warn_unsupported_ops)
+            .uncalled_modules_warnings(self._enable_warn_uncalled_mods)
+            .tracer_warnings(self._warn_trace)
         )
-        analyzer._warn_skipped = self._warn_skipped
-        analyzer._warn_trace = self._warn_trace
 
-        return analyzer
-
-    def tracer_warnings(self, mode: str) -> None:
+    def tracer_warnings(self, mode: str) -> "JitModelAnalysis":
         """
         Sets which warnings to print when tracing the graph to calculate
         statistics. There are three modes. Defaults to 'no_tracer_warning'.
+        Allowed values are:
 
-        Modes:
-        'all' : keeps all warnings raised while tracing
-        'no_tracer_warning' : suppress torch.jit.TracerWarning only
-        'none' : suppress all warnings raised while tracing
+        * 'all' : keeps all warnings raised while tracing
+        * 'no_tracer_warning' : suppress torch.jit.TracerWarning only
+        * 'none' : suppress all warnings raised while tracing
 
         Args:
-            mode (str) : warning mode. In ['all', 'no_tracer_warning', 'none'].
+            mode (str) : warning mode in one of the above values.
         """
         assert mode in [
             "all",
@@ -385,25 +422,60 @@ class JitModelAnalysis(object):
             "none",
         ], "Unrecognized trace warning mode."
         self._warn_trace = mode
+        return self
 
-    def skipped_ops_warnings(self, enabled: bool) -> None:
+    def unsupported_ops_warnings(self, enabled: bool) -> "JitModelAnalysis":
         """
-        Sets if warnings from skipped operators are shown. Defaults
-        to True. Counts of skipped operators may be obtained from
-        .skipped_ops(module) regardless of this setting.
+        Sets if warnings for unsupported operators are shown. Defaults
+        to True. Counts of unsupported operators may be obtained from
+        :meth:`unsupported_ops` regardless of this setting.
 
         Args:
-            enabled (bool) : Set to 'True' to show skipped operator
+            enabled (bool) : Set to 'True' to show unsupported operator
                 warnings.
         """
-        self._warn_skipped = enabled
+        self._enable_warn_unsupported_ops = enabled
+        return self
 
-    def _warn_skipped_ops(self, skipped_ops: typing.Counter[str]) -> None:
-        if not self._warn_skipped:
+    def uncalled_modules_warnings(self, enabled: bool) -> "JitModelAnalysis":
+        """
+        Sets if warnings from uncalled submodules are shown. Defaults to true.
+        A submodule is considered "uncalled" if it is never called during
+        tracing. This may be because it is actually unused, or because it is
+        accessed via calls to ``.forward()`` or other methods of the module.
+        The set of uncalled modules may be obtained from
+        :meth:`uncalled_modules` regardless of this setting.
+
+        Args:
+            enabled (bool) : Set to 'True' to show warnings.
+        """
+        self._enable_warn_uncalled_mods = enabled
+        return self
+
+    def _warn_unsupported_ops(self, ops: typing.Counter[str]) -> None:
+        if not self._enable_warn_unsupported_ops:
             return
         logger = logging.getLogger(__name__)
-        for op, freq in skipped_ops.items():
-            logger.warning("Skipped operation {} {} time(s)".format(op, freq))
+        for op, freq in ops.items():
+            logger.warning(
+                "Unsupported operator {} encountered {} time(s)".format(op, freq)
+            )
+
+    def _warn_uncalled_mods(self, uncalled_mods: Set[str]) -> None:
+        if not self._enable_warn_uncalled_mods or not uncalled_mods:
+            return
+        uncalled_mods = {x for x in uncalled_mods if self._has_forward(x)}
+
+        logger = logging.getLogger(__name__)
+        logger.warning(
+            "The following submodules of the model were never "
+            "called during the trace of the graph. They may be "
+            "unused, or they were accessed by direct calls to "
+            ".forward() or via other python methods. In the latter "
+            "case they will have zeros for statistics, though their "
+            "statistics will still contribute to their parent calling "
+            "module.\n" + ", ".join(sorted(uncalled_mods))
+        )
 
     def _get_aliases(self, model: nn.Module) -> Dict[Union[str, nn.Module], str]:
         aliases = {}
@@ -413,13 +485,11 @@ class JitModelAnalysis(object):
             aliases[name] = aliases[module]
         return aliases
 
-    def _analyze(
-        self,
-    ) -> Tuple[Dict[str, typing.Counter[str]], Dict[str, typing.Counter[str]]]:
+    def _analyze(self) -> "Statistics":
         # Don't calculate if results are already stored.
-        counts, skipped_ops = self._counts, self._skipped_ops
-        if counts is not None and skipped_ops is not None:
-            return counts, skipped_ops
+        stats = self._stats
+        if stats is not None:
+            return stats
 
         with warnings.catch_warnings():
             if self._warn_trace == "none":
@@ -430,37 +500,67 @@ class JitModelAnalysis(object):
 
         # Assures even modules not in the trace graph are initialized to zero count
         counts = {}
-        skipped_ops = {}
+        unsupported_ops = {}
         # We don't need the duplication here, but self._model.named_modules()
         # gives slightly different results for some wrapped models.
         for _, mod in _named_modules_with_dup(self._model):
             name = self._aliases[mod]
             counts[name] = Counter()
-            skipped_ops[name] = Counter()
+            unsupported_ops[name] = Counter()
 
+        all_seen = set()
         for node in graph.nodes():
             kind = node.kind()
             scope_names = node.scopeName().split("/")
+            all_seen.update(scope_names)
             if kind not in self._op_handles:
-                if kind in self.ignored_ops:
+                # ignore all prim:: operators
+                if kind in self._ignored_ops or kind.startswith("prim::"):
                     continue
 
-                seen = set()
-                for name in scope_names:
-                    if name not in seen:
-                        skipped_ops[name][kind] += 1
-                        seen.add(name)
+                for name in set(scope_names):
+                    unsupported_ops[name][kind] += 1
             else:
                 inputs, outputs = list(node.inputs()), list(node.outputs())
                 op_counts = self._op_handles[kind](inputs, outputs)
+                if isinstance(op_counts, Number):
+                    op_counts = Counter({self._simplify_op_name(kind): op_counts})
 
-                seen = set()  # Assures an op contributes at most once to a module
-                for name in scope_names:
-                    if name not in seen:
-                        counts[name] += op_counts
-                        seen.add(name)
+                # Assures an op contributes at most once to a module
+                for name in set(scope_names):
+                    counts[name] += op_counts
 
-        self._counts = counts
-        self._skipped_ops = skipped_ops
-        self._warn_skipped_ops(skipped_ops[""])
-        return counts, skipped_ops
+        uncalled_mods = set(self._aliases.values()) - all_seen
+        stats = Statistics(
+            counts=counts, unsupported_ops=unsupported_ops, uncalled_mods=uncalled_mods
+        )
+        self._stats = stats
+        self._warn_unsupported_ops(unsupported_ops[""])
+        self._warn_uncalled_mods(uncalled_mods)
+        return stats
+
+    def _simplify_op_name(self, full_op_name: str) -> str:
+        """
+        Get simplified name of the op without the preceding namespace, e.g.
+        aten::batch_norm -> batch_norm
+        """
+        p = full_op_name.find("::")
+        if p != -1:
+            return full_op_name[p + 2 :]
+        else:
+            return full_op_name
+
+    def _has_forward(self, mod_name: str) -> bool:
+        # Whether the module has a valid forward method.
+        # Modules without forward are not expected to get called
+        # and therefore should not produce "uncalled" warnings
+        module = self._named_modules.get(mod_name)
+        if module is None:
+            return False
+        module_type = type(module)
+        # Containers are not meant to be called anyway (they don't have forward)
+        no_forward_mods = {nn.ModuleList, nn.ModuleDict, nn.Module}
+        for mod in no_forward_mods:
+            if module_type.forward is mod.forward:  # pyre-ignore[16]
+                return False
+        return True
