@@ -713,7 +713,8 @@ class CropTransform(Transform):
         for polygon in polygons:
             polygon = geometry.Polygon(polygon).buffer(0.0)
             # polygon must be valid to perform intersection.
-            assert polygon.is_valid, polygon
+            if not polygon.is_valid:
+                continue
             cropped = polygon.intersection(crop_box)
             if cropped.is_empty:
                 continue
@@ -753,6 +754,7 @@ class PadTransform(Transform):
         orig_w: Optional[int] = None,
         orig_h: Optional[int] = None,
         pad_value: float = 0,
+        seg_pad_value: int = 0,
     ):
         """
         Args:
@@ -760,7 +762,8 @@ class PadTransform(Transform):
             x1, y1: number of padded pixels on the right and bottom
             orig_w, orig_h: optional, original width and height.
                 Needed to make this transform invertible.
-            pad_value: the padding value
+            pad_value: the padding value to the image
+            seg_pad_value: the padding value to the segmentation mask
         """
         super().__init__()
         self._set_attributes(locals())
@@ -777,6 +780,18 @@ class PadTransform(Transform):
             constant_values=self.pad_value,
         )
 
+    def apply_segmentation(self, img):
+        if img.ndim == 3:
+            padding = ((self.y0, self.y1), (self.x0, self.x1), (0, 0))
+        else:
+            padding = ((self.y0, self.y1), (self.x0, self.x1))
+        return np.pad(
+            img,
+            padding,
+            mode="constant",
+            constant_values=self.seg_pad_value,
+        )
+
     def apply_coords(self, coords):
         coords[:, 0] += self.x0
         coords[:, 1] += self.y0
@@ -804,7 +819,9 @@ class BlendTransform(Transform):
         ``src_weight * src_image + dst_weight * dst_image``
 
         Args:
-            src_image (ndarray): Input image is blended with this image
+            src_image (ndarray): Input image is blended with this image.
+                The two images must have the same shape, range, channel order
+                and dtype.
             src_weight (float): Blend weighting of src_image
             dst_weight (float): Blend weighting of dst_image
         """
